@@ -1,19 +1,23 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2010-2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2010-2012 -- leonerd@leonerd.org.uk
 
 package Tangence::ObjectProxy;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 use Carp;
 
 use Tangence::Constants;
 
+use Tangence::Meta::Type;
+
+use constant TYPE_U8 => Tangence::Meta::Type->new( "u8" );
+
 use Scalar::Util qw( weaken );
 
 =head1 NAME
@@ -683,7 +687,7 @@ sub handle_request_UPDATE
    my ( $message ) = @_;
 
    my $prop  = $message->unpack_str();
-   my $how   = $message->unpack_typed( "u8" );
+   my $how   = $message->unpack_typed( TYPE_U8 );
 
    my $pdef = $self->can_property( $prop ) or return;
    my $type = $pdef->{type};
@@ -723,7 +727,7 @@ sub _update_property_hash
    my ( $p, $type, $how, $message ) = @_;
 
    if( $how == CHANGE_SET ) {
-      my $value = $message->unpack_typed( "dict($type)" );
+      my $value = $message->unpack_typed( Tangence::Meta::Type->new( dict => $type ) );
       $p->{cache} = $value;
       $_->{on_set} and $_->{on_set}->( $p->{cache} ) for @{ $p->{cbs} };
    }
@@ -749,7 +753,7 @@ sub _update_property_queue
    my ( $p, $type, $how, $message ) = @_;
 
    if( $how == CHANGE_SET ) {
-      my $value = $message->unpack_typed( "list($type)" );
+      my $value = $message->unpack_typed( Tangence::Meta::Type->new( list => $type ) );
       $p->{cache} = $value;
       $_->{on_set} and $_->{on_set}->( $p->{cache} ) for @{ $p->{cbs} };
    }
@@ -774,7 +778,7 @@ sub _update_property_array
    my ( $p, $type, $how, $message ) = @_;
 
    if( $how == CHANGE_SET ) {
-      my $value = $message->unpack_typed( "list($type)" );
+      my $value = $message->unpack_typed( Tangence::Meta::Type->new( list => $type ) );
       $p->{cache} = $value;
       $_->{on_set} and $_->{on_set}->( $p->{cache} ) for @{ $p->{cbs} };
    }
@@ -822,7 +826,7 @@ sub _update_property_objset
 
    if( $how == CHANGE_SET ) {
       # Comes across in a LIST. We need to map id => obj
-      my $objects = $message->unpack_typed( "list($type)" );
+      my $objects = $message->unpack_typed( Tangence::Meta::Type->new( list => $type ) );
       $p->{cache} = { map { $_->id => $_ } @$objects };
       $_->{on_set} and $_->{on_set}->( $p->{cache} ) for @{ $p->{cbs} };
    }
