@@ -1,14 +1,15 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2012 -- leonerd@leonerd.org.uk
 
 package Tangence::Stream;
 
 use strict;
 use warnings;
+use 5.010; # //
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 use Carp;
 
@@ -32,6 +33,7 @@ my %REQ_METHOD = (
 
    MSG_GETROOT,     'handle_request_GETROOT',
    MSG_GETREGISTRY, 'handle_request_GETREGISTRY',
+   MSG_INIT,        'handle_request_INIT',
 );
 
 =head1 NAME
@@ -248,6 +250,30 @@ sub respond
    }
 }
 
+sub respondERROR
+{
+   my $self = shift;
+   my ( $token, $string ) = @_;
+
+   $self->respond( $token, Tangence::Message->new( $self, MSG_ERROR )
+      ->pack_str( $string )
+   );
+}
+
+=head2 $ver = $stream->minor_version
+
+Returns the minor version negotiated by the C<MSG_INIT> / C<MSG_INITED>
+initial message handshake.
+
+=cut
+
+sub minor_version
+{
+   my $self = shift;
+   ( $self->{tangence_minor_version} ) = @_ if @_;
+   return $self->{tangence_minor_version} // 0;
+}
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
